package de.opalpiraten.scenepainter.sys;
import de.opalpiraten.scenepainter.gui.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import java.util.prefs.*;
import de.opalpiraten.scenepainter.tools.*;
import com.threerings.resource.*;

/**
 * The Core of the (GUI) Application. This class is responsible for starting
 * up and registering global structures.
 */ 
public class Core implements WindowListener, ActionListener {

  /**
   * Version String (as shown in the window title)
   */
  public static final String VERSION ="ScenePainter v2.3";
  
  /**
   * The Workspace
   */
  public static Workspace workspace;
  
  /**
   * Access to the nenya resources
   */
  public static NenyaBrowser nenyabrowser;
  
  /**
   * Frame, displaying the application
   */
  public static JFrame frame;
  
  /**
   * Start the program in interactive GUI mode
   */
  public Core() {
    workspace = new Workspace();
    Project project = new Project(Project.DEFAULTNAME);
    workspace.setProject(project);
    workspace.setScene(Util.newScene());
    frame = new JFrame();
    frame.setTitle(VERSION);
    frame.setJMenuBar(new Menus());
    frame.setSize(java.awt.Toolkit.getDefaultToolkit().getScreenSize());
    frame.setContentPane(workspace);
    frame.setVisible(true);
    frame.addWindowListener(this);
    frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    Menus.quit.addActionListener(this);
    
    Menus.addToConfig(PathSettings.getHook());
    Menus.loadTools();
  }
  
  /**
   * Call this method to properly shut down the Application
   */
  public static void quit() {
    try {
      if (workspace.getScene().dirty) {
        if (JOptionPane.YES_OPTION != JOptionPane.showInternalOptionDialog(workspace,Dict.get("sys.core.unsaved"),Dict.get("sys.core.unsaved.title"),JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,null,null)) {
          return;
        }
      }
    }
    catch (Exception exp) {
      // Getting here means, there cannot possibly be unsaved work and
      // simply quitting is a save thing to do.
    }
    System.exit(0);
  }
  
  /**
   * Try to load the nenyabrowser
   * @param base base directory to load from
   * @param obs Observer to be notfied of progress (may be null).
   * @return true nenyabrowser was successfully loaded
   */
  private static boolean loadBrowser(File base,ResourceManager.InitObserver obs) {
    try {
      NenyaDescriptor nd = ConfiguredDescriptor.getDescriptor(base);
      if (nd == null) throw new IllegalArgumentException();
      nenyabrowser = new NenyaBrowser(nd,obs);
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }
  
  public void windowClosing(WindowEvent e) { quit();}
  public void focusLost(FocusEvent e) {}
  public void windowOpened(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}


  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if (src == Menus.quit) quit();
  }
  
  
  public static void main (String[] args) {
    Preferences cfg = Preferences.userNodeForPackage(Core.class);

    String loc = cfg.get("language",null);
    if (loc!=null) Locale.setDefault(new Locale(loc));
    
    Dict.addBundle(ClassLoader.getSystemClassLoader());
    File base = null;
    
    if (args.length!=0) {
      // Try to get the nenya directory via cmdline argument first. A user,
      // using the commandline is assumed to know what s/he is doing, so
      // wipe out the configuration first.
      base= new File(args[0]);
      cfg.remove("nenyabase");
      loadBrowser(base,null);
    }
    else {
      // If nothing was submitted via cmdline, maybe the location can be
      // discovered via the preferences, so try that before asking the user
      // explicitly.
      String name = cfg.get("nenyabase",null);
      if (name!=null) { 
        base = new File(name);
        loadBrowser(base,null);
      }
    }
    
    // Block until a nenyabrowser is instantiated
    while(nenyabrowser==null) {
      // No base directory so far. Present the user with a JFileChooser
      // until either s/he selects an appropriate location or gives up
      // and reads the docs.
      Inspector ins = new Inspector();
      JFileChooser jfc = new JFileChooser();
      jfc.setDialogTitle(Dict.get("sys.core.choose"));
      jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      jfc.setAcceptAllFileFilterUsed(false);
      jfc.setSelectedFile(new File(System.getProperty("user.dir")));
      jfc.setAccessory(ins);
      jfc.addPropertyChangeListener(ins);
      if (jfc.showOpenDialog(null)!=JFileChooser.APPROVE_OPTION) System.exit(0);
      base=jfc.getSelectedFile();
      
      if (!loadBrowser(base,null)) {
        // Give the user the chance to try again or quit
        int response= JOptionPane.showConfirmDialog(null,Dict.get("sys.core.nonenya"),Dict.get("sys.core.nonenya.title"),JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE,null);
        if (response==JOptionPane.NO_OPTION) System.exit(0);
      }
    }
    
    // Reaching here means, a nenyabrowser was established -> write back
    // preferences and load the rest of the program.
    
    cfg.put("nenyabase",base.getAbsolutePath());
    new Core();
  }
}
